#include "asf.h"
#include <avr/interrupt.h>
#include "stdio.h"
#include <string.h>
#include <stdlib.h>


#define FOSC							BOARD_EXTERNAL_CLK		/* FOSC = XTAL pin frequency */


static int uart0_tx(char c, FILE *stream);
static void uart0_rx(FILE *stream);

static FILE mystdio = FDEV_SETUP_STREAM(uart0_tx, uart0_rx, _FDEV_SETUP_RW);


/* CAUTION :
*           unsigned int : 16bits in ATmega
*           uint32_t     : 32bits in ATmega
*/

static int uart0_tx(char c, FILE *stream)
{
	if(c == '\n')
	{
		while((UCSR0A & (1 << UDRE0)) == 0);
		UDR0 = 0x0D;	// carriage return
	}

	while((UCSR0A & (1 << UDRE0)) == 0);
	UDR0 = c;


	return 0;
}




static void uart0_rx(FILE *stream)
{
	char data;

	while (!(UCSR0A & (1 << RXC0)));

	data = UDR0;
}




void USART0_Printf_Init(uint32_t baudrate)
{
	float f_value;
	uint32_t ubr_reg;

	sysclk_enable_peripheral_clock(&UCSR0A);

	/* Set baud rate */
	/* Caution : Expression FOSC / ( 16 * baudrate) - 1 may cause data crack. */

	//f_value = ((float)(FOSC / 16.) / baudrate - 1);
	f_value = ((float)FOSC / (16. * baudrate) - 1.);

	ubr_reg = (uint32_t)f_value;
	if(f_value - ubr_reg > 0.5)
		ubr_reg++;

	UCSR0A = 0;
	UCSR0B = 0;

	/* Set frame format: 8data, 1stop bit */
	UCSR0C = (0 << USBS0) | (3 << UCSZ00);

	UBRR0H = (unsigned char)((ubr_reg >> 8) & 0xFF);
	UBRR0L = (unsigned char)(ubr_reg & 0xFF);

	/* Enable receiver and transmitter #1. */
	UCSR0B = (1 << RXEN0) | (1 << TXEN0);

	// Assign device(mystdio) to stdout & stdin.
	stdout = stdin = &mystdio;

	delay_ms(10);
}



ISR(USART0_RX_vect)
{
	char rcv_data;

	while(!(UCSR0A & (1 << RXC0)));
	rcv_data = UDR0;

	uart0_tx(rcv_data, NULL);
}

