/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
/*
 * Support and FAQ: visit <a href="http://www.atmel.com/design-support/">Atmel Support</a>
 */
#include <asf.h>
#include <stdio.h>

#include "drv_usart.h"


#define ADC_VOLTAGE_REF				5			// AREF 5V


static void ADC6_SEI_Init(uint8_t auto_flag);
static void ADC6_AutoTriggering_Test(void);


int main(void)
{
	uint8_t adc_auto_flag = 1;
	uint32_t baudrate = 115200;
	uint16_t count;
	char key;

	/* Insert system clock initialization code here (sysclk_init()). */
	sysclk_init();
	sysclk_set_prescalers(SYSCLK_PSDIV_1);

	board_init();

	USART0_Printf_Init(baudrate);

	delay_ms(100);

	printf("CPU Clock = %lu\n", F_CPU);
	printf("baudrate = %lu\n", baudrate);

	ADC6_SEI_Init(adc_auto_flag);
	delay_ms(10);

	sei();

	while(1)
	{
		printf("ADC %s Test Ready? (Y/N) : ", (adc_auto_flag ? "Auto" : "Manual"));
		scanf("%c", &key);
		printf("\r\n");

		if((key == 'Y') || (key == 'y'))
			break;

		if((key == 'N') || (key == 'n'))
		{
			printf("Test canceled.\r\n");
			while(1);
		}
	}

	/* Insert application code here, after the board has been initialized. */
	while(1)
	{
		ADC6_AutoTriggering_Test();
		delay_ms(100);
	}
}




/* ADC6 Single Ended Input Init : Set ADC Auto Trigger Enable. */

static void ADC6_SEI_Init(uint8_t auto_flag)
{
	sysclk_enable_peripheral_clock(&ADC);

	// Division Factor 128 : 16MHz(fclk) / 128 = 125kHz
	ADCSRA = (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);

	// ADC Auto Trigger Enable
	if(auto_flag == 1)
		ADCSRA |= (1 << ADATE);

	// Free Running mode
	ADCSRB = (0 << ADTS2) | (0 << ADTS1) | (0 << ADTS0);

	//ADC6
	ADMUX = (0 << MUX3) | (1 << MUX2) | (1 << MUX1) | (0 << MUX0);

	//Set voltage reference to AREF
	ADMUX |= (0 << REFS1) | (0 << REFS0);

	// Right Adjust Result.
	ADMUX |= (0 << ADLAR);

	// Enable ADC
	ADCSRA |= (1 << ADEN);

	// ADC Start Conversion
	ADCSRA |= (1 << ADSC);
}




static void ADC6_AutoTriggering_Test(void)
{
	uint32_t adc_sum = 0;
	uint32_t count;
	float adc_value;


	for(count = 0; count < 16; count++)
	{

		delay_ms(1);

#if 0	// This also works.
		adc_sum += ADCW;
#else
		/* Caution : Firstly read ADCL and then ADCH. See 29.9.3 in the ATmega328PB datasheet. */
		adc_sum += ADCL;
		adc_sum += ((ADCH & 0x03) << 8);
#endif
	}

	adc_sum >>= 4;

	//printf("adc_sum %d\n", adc_sum);

	adc_value = (adc_sum * ADC_VOLTAGE_REF) / 1024.;
	printf("voltage = %f V\n", adc_value);
}
